'' =================================================================================================
''
''   File....... jm_encoder_1x.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2012-2021 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 01 JAN 2021
''
'' =================================================================================================

{

  Basic quadrature encoder driver.

  Note that some detented encoders will return 4 counts per detent.
  -- set det4x to true

}


con { fixed io pins }

  RX1  = 31                                                     ' program / debug
  TX1  = 30

  SDA1 = 29                                                     ' eeprom / i2c
  SCL1 = 28


con

  #true, DETENT, NO_DETENT                                      ' "click" or "no click"

  #0, PRESSED, RELEASED                                         ' for active-low button


var

  long  cog                                                     ' pasm cog

  long  btnpin                                                  ' button input (-1 if not used)
  long  mod4x                                                   ' 4x per detent modification
  long  qecounts                                                ' encoder counts (raw)
  long  lolimit                                                 ' for range limiting
  long  hilimit

  long  ms1tix                                                  ' ticks in 1ms


pub null

'' This is not a top-level object


pub start(a, b, btn, d4x, preset, lo, hi)

'' Start the encoder object
'' -- continuous count mode
'' -- a & b (clock & data) are encoder inputs (active-low)
'' -- btn is the button input (active-low, use -1 if not used)
'' -- set d4x to true if each "click" is 4 counts
'' -- preset is initial value for encoder
'' -- lo & hi are limit values for encoder

  stop                                                          ' stop if already running

  if (a == b)                                                   ' validate pins
    return false

  longmove(@btnpin, @btn, 5)                                    ' copy setup parameters

  dira[a] := 0                                                  ' clear pins in this cog
  dira[b] := 0
  if (btn => 0)
    dira[btn] := 0

  qecounts.byte[0] := a                                         ' prep pins for cog
  qecounts.byte[1] := b

  cog := cognew(@encoder1, @qecounts) + 1                       ' start encoder cog

  if (cog == 0)                                                 ' failure to launch...
    return false

  repeat
  until (qecounts == 0)                                         ' let cog get setup

  if (d4x)                                                      ' set detent adjustment
    mod4x := 2
  else
    mod4x := 0

  fix_limits                                                    ' force lolimit < hilimit
  set_value(preset)                                             ' preset encoder value

  ms1tix := clkfreq / 1000                                      ' for debounce delays

  return cog


pub stop

'' Unload encoders cog

  if (cog)
    cogstop(cog-1)
    cog := 0

  longfill(@btnpin, 0, 5)


pub clear

'' Set encoder to 0
'' -- do not use if 0 not in lolimit..hilimit range

  set_value(0)


pub set_value(preset)

'' Set encoder to preset value

  preset := lolimit #> preset <# hilimit                        ' force into range

  qecounts := preset << mod4x                                   ' update internal count

  return preset


pub set_range(lo, hi)

'' Update encoder range
'' -- will constrict current value if out of new range

  longmove(@lolimit, @lo, 2)                                    ' get new range
  fix_limits

  result := value                                              ' update encoder if needed


pub configure(preset, lo, hi)

'' Configure encoder range and current value

  set_range(lo, hi)                                             ' update new range
  set_value(preset)                                             ' update current value

  result := value


pub value | raw

'' Return encoder value
'' -- truncated to lolimit..hilimit

  raw := qecounts ~> mod4x                                      ' get raw value

  if (raw < lolimit)                                            ' limit range
    result := set_value(lolimit)
  elseif (raw > hilimit)
    result := set_value(hilimit)
  else
    result := raw


pub check_press(ms) : t

'' Debounce encoder button for delay milliseconds
'' -- return true for pressed, false for not pressed

  if ((cog) and (btnpin => 0))                                  ' button used?
    repeat while (ina[btnpin] == PRESSED)                       ' pressed?
      waitcnt(cnt + ms1tix)                                     '  wait 1ms
      if (++t => ms)                                            '  bump and check timer
        return true

  return false


pub force_release(ms) : t

'' Holds until button is clear ms milliseconds

  if ((cog) and (btnpin => 0))                                  ' button used?
    repeat
      if (ina[btnpin] == RELEASED)                              ' if button not pressed
        waitcnt(cnt + ms1tix)                                   '  wait 1ms
        if (++t => ms)                                          '  bump timer and check
          return
      else
        t := 0                                                  '  else reset timer


pri fix_limits | tmp

'' Force lolimit to be less than hilimit

  if (lolimit > hilimit)                                        ' swap lo & hi if needed
    tmp     := lolimit
    lolimit := hilimit
    hilimit := tmp


dat

                org       0

encoder1        rdlong    t1, par                               ' read pins
                mov       t2, t1                                ' make a copy

setup_enc       and       t1, #$1F                              ' isolate a pin
                mov       amask, #1                             ' convert to mask
                shl       amask, t1

                shr       t2, #8                                ' move b pin to byte 0
                and       t2, #$1F                              ' isolate b pin
                mov       bmask, #1                             ' convert to mask
                shl       bmask, t2

                call      #sample                               ' get starting state
                mov       oldscan, newscan                      ' save
                mov       t1, #0                                ' clear counter
                wrlong    t1, par                               ' tell hub we're ready


enc_loop        call      #sample                               ' get new state
                cmp       oldscan, newscan              wz      ' change?
    if_e        jmp       #enc_loop                             ' in no, try again

                mov       t1, oldscan                           ' yes
                shl       t1, #1
                xor       t1, newscan                           ' compare new A to old B
                test      t1, #%10                      wc      ' 1 = different, 0 = same
                rdlong    t2, par                               ' get current value
    if_c        adds      t2, #1                                ' different, +
    if_nc       subs      t2, #1                                ' same, -
                wrlong    t2, par                               ' update hub
                mov       oldscan, newscan                      ' move new to old for next check
                jmp       #enc_loop


' sample encoder pins
' -- updates 'newscan' as: %AB

sample          mov       t1, ina                               ' snap-shot inputs
                mov       newscan, #0                           ' clear new result
                test      t1, amask                     wc      ' get a level into c
                muxc      newscan, #%10                         ' move to newscan.1
                test      t1, bmask                     wc      ' get a level into c
                muxc      newscan, #%01                         ' move to newscan.1
sample_ret      ret


' ----------------------------------------------------------------------------------------

amask           res       1                                     ' encoder pin masks
bmask           res       1

newscan         res       1                                     ' input scans
oldscan         res       1

t1              res       1                                     ' work vars
t2              res       1

                fit       496


con { license }

{{

  MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

}}