'' =================================================================================================
''
''   File....... jm_quadrature_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2020-2021 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 01 JAN 2021
''
''   {$P1}
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { fixed io pins }

  RX1     = 31  { I }                                           ' serial / programming
  TX1     = 30  { O }

  SDA1    = 29  { I/O }                                         ' i2c / eeprom
  SCL1    = 28  { I/O }


con { app io pins }

  ENC_BTN =  6  { I }                                           ' active-low encoder connections
  ENC_B   =  5  { I }                                           ' encoder data
  ENC_A   =  4  { I }                                           ' encoder clock


con { terminal }

  BR_TERM  = 115_200                                            ' terminal baud rate

  #0, T_PST, T_ANSI                                             ' terminal types

  T_TYPE = T_ANSI


con

  #true,  ON, OFF                                               ' digital control
  #false, NO, YES

  LO = -100                                                     ' limits for encoder
  HI =  100


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  prng : "jm_prng"                                              '   pseudo-random number generator
  io   : "jm_io_basic"                                          '   essential io
  quad : "jm_encoder_1x"                                        ' * quadrature encoder driver
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
  ansi : "jm_ansi"                                              '   ANSI terminal control sequences

' * uses cog when loaded


dat

  Banner        byte    "P1 Quadrature Encoder Demo", 0

  IsPressed     byte    "Pressed     ", 0
  NotPressed    byte    "Not Pressed ", 0


pub main | t, ev, p_str, k

  setup

  wait_for_terminal(true)

  t := cnt                                                      ' start loop timer
  repeat
    ev := quad.value                                            ' get counts ("clicks")

    if (T_TYPE == T_PST)
      term.fstr2(string("%c%s\r\r"), term#HOME, @Banner)
      term.fstr1(string("Value.... %d   \r"), ev)
    else
      term.str(ansi.home)
      term.fstr3(string("%s%s%s\r\r"), ansi.fgnd(ansi#BR_CYAN), @Banner, ansi.normal)
      term.str(string("Value.... "))
      if (ev < 0)
        term.str(ansi.fgnd(ansi#BR_RED))
      elseif (ev > 0)
        term.str(ansi.fgnd(ansi#BR_GREEN))
      else
        term.str(ansi.normal)
      term.fstr2(string("%d%s   \r"), ev, ansi.normal)

    if (quad.check_press(5))
      p_str := @IsPressed
    else
      p_str := @NotPressed

    term.fstr1(string("Button... %s\r"), p_str)

    k := term.rxcheck
    case k
      "x", "X" : quad.clear                                     ' reset encoder to 0
      "r"      : quad.set_range(-50, 50)                        ' shrink range
      "R"      : quad.set_range( LO, HI)                        ' expand range

    waitcnt(t += (50 * MS_001))                                 ' loop runs every 50ms


pub wait_for_terminal(clear)

'' Allow user to open terminal and press key when ready

  term.rxflush
  term.rx
  if (clear)
    if (T_TYPE == T_PST)
      term.tx(term#CLS)
    else
      term.str(ansi.hide_cursor)
      term.str(ansi.home)
      term.str(ansi.cls)


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing & delays

  prng.seed(-cnt, -cnt ~> 2, $CAFE_BABE, cnt <- 2, cnt)         ' seed randomizer

  io.start(0, 0)                                                ' clear all pins (master cog)

  quad.start(ENC_A, ENC_B, ENC_BTN, true, 0, LO, HI)            ' start quadrature encoder *

  term.tstart(BR_TERM)                                          ' start serial for terminal *


con { license }

{{

  MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

}}